/*============================================================================
*
*                    Code_Saturne version 1.3
*                    ------------------------
*
*
*     This file is part of the Code_Saturne Kernel, element of the
*     Code_Saturne CFD tool.
*
*     Copyright (C) 1998-2007 EDF S.A., France
*
*     contact: saturne-support@edf.fr
*
*     The Code_Saturne Kernel is free software; you can redistribute it
*     and/or modify it under the terms of the GNU General Public License
*     as published by the Free Software Foundation; either version 2 of
*     the License, or (at your option) any later version.
*
*     The Code_Saturne Kernel is distributed in the hope that it will be
*     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
*     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with the Code_Saturne Kernel; if not, write to the
*     Free Software Foundation, Inc.,
*     51 Franklin St, Fifth Floor,
*     Boston, MA  02110-1301  USA
*
*============================================================================*/

#ifndef __CS_MAILLAGE_H__
#define __CS_MAILLAGE_H__

/*============================================================================
 * Structure principale associe  un maillage
 *============================================================================*/

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*----------------------------------------------------------------------------
 *  Fichiers `include' BFT ou FVM
 *----------------------------------------------------------------------------*/

#include "fvm_defs.h"

/*----------------------------------------------------------------------------
 *  Fichiers `include' locaux
 *----------------------------------------------------------------------------*/

#include "cs_base.h"
#include "cs_perio.h"


/*=============================================================================
 * Dfinitions de macros
 *============================================================================*/

/* Type de voisinage tendu, recu de l'Enveloppe */

#define CS_MAILLAGE_TYPE_VOISET_SANS       0 /* Pas de voisinage tendu      */
#define CS_MAILLAGE_TYPE_VOISET_DANS_HALO  1 /* Voisinage tendu trait
                                                     dans le halo standard   */
#define CS_MAILLAGE_TYPE_VOISET_SEPARE     2 /* Voisinage tendu spar      */


/*============================================================================
 * Dfinitions de types
 *============================================================================*/

/* Structure associe  un maillage */

typedef struct {

  /* Indicateurs globaux */

  cs_int_t   dim;                  /* Dimension de l'espace */
  cs_int_t   num_dom;              /* Numro du domaine local */
  cs_int_t   nbr_dom;              /* Nombre global de domaines */
  cs_int_t   nbr_per;              /* Nombre de priodicits */
  cs_int_t   ind_type_voiset;      /* Type de voisinage tendu :
                                      = CS_VOISET_TYPE_SANS
                                            : pas de voisinage tendu
                                      = CS_VOISET_TYPE_DANS_HALO
                                            : voisinage tendu dans le halo std
                                      = CS_VOISET_TYPE_SEPARE
                                            : voisinage tendu spar  */

  /* Dimensions locales */

  cs_int_t   nbr_cel;              /* Nombre de cellules du maillage */
  cs_int_t   nbr_fac;              /* Nombre de faces internes du maillage */
  cs_int_t   nbr_fbr;              /* Nombre de faces de bord du maillage */
  cs_int_t   nbr_som;              /* Nombre de sommets du maillage */

  cs_int_t   lng_fac_som;          /* Taille connectivit faces -> sommets */
  cs_int_t   lng_fbr_som;          /* Taille de la connectivit
                                      faces de bord -> sommets */

  cs_int_t   nbr_cel_etendu;       /* Nombre de cellules tendu
                                      (nbr_cel + nbr_cel_fac_par) */
  cs_int_t   nbr_cel_fac_par;      /* Nombre de cellules "fantmes" */
  cs_int_t   nbr_dom_fac_par;      /* Nombre de domaines sur frontire
                                      parallle */
  cs_int_t   nbr_cel_fac_par_voiset;
                                   /* Nombre de cellules "fantmes" pour le
                                      voisinage tendu spar  */
  cs_int_t   nbr_dom_fac_par_voiset;
                                   /* Nombre de domaines sur frontire
                                      parallle pour le
                                      voisinage tendu spar  */
  cs_int_t   nbr_val_connect_voiset_cel_cel_dom;
                                   /* Longueur lelvse du tableau
                                      val_connect_voiset_cel_cel_dom des valeurs
                                      de la connectivit du voisinage tendu
                                      On pourrait s'affranchir de cette variable
                                      (dont la valeur est gale 
                                      pos_connect_voiset_cel_cel_dom[nbr_cel]-1)
                                      dans la mesure o, actuellement,
                                        - le tableau des positions
                                          pos_connect_voiset_cel_cel_dom
                                          est envoy par l'enveloppe avant
                                          le tableau des valeurs
                                          val_connect_voiset_cel_cel_dom
                                        - le tableau des valeurs est allou
                                          en C uniquement et pas en FORTRAN   */

  /* Dimensions globales */

  cs_int_t   nbr_cel_glob;         /* Nombre de cellules global */
  cs_int_t   nbr_fac_glob;         /* Nombre de faces internes global */
  cs_int_t   nbr_fbr_glob;         /* Nombre de faces de bord global */
  cs_int_t   nbr_som_glob;         /* Nombre de sommets global */

  /* Structures locales */

  cs_real_t  *coo_som;             /* Coordonnes des sommets */
  cs_int_t   *fac_cel;             /* Connectivit faces internes -> cellules */
  cs_int_t   *fbr_cel;             /* Connectivit faces de bord -> cellules */
  cs_int_t   *pos_fac_som;         /* Positions de dpart dans la
                                      connectivit faces internes -> sommets */
  cs_int_t   *val_fac_som;         /* Connectivit faces internes -> sommets */
  cs_int_t   *pos_fbr_som;         /* Positions de dpart dans la
                                      connectivit faces de bord -> sommets */
  cs_int_t   *val_fbr_som;         /* Connectivit faces de bord -> sommets */

  /* Numrotations globales */

  fvm_gnum_t  *num_cel;            /* Numros globaux des cellules */
  fvm_gnum_t  *num_fac;            /* Numros globaux des faces internes
                                      (restent en numrot. locale initiale) */
  fvm_gnum_t  *num_fbr;            /* Numros globaux des faces de bord
                                      (restent en numrot. locale initiale) */
  fvm_gnum_t  *num_som;            /* Numros globaux des sommets */

  /* Structures auxiliaires pour paralllisme et priodicit */

  cs_int_t   *pos_dom_fac_par;     /* Pointeurs sur blocs de cellules "fantmes"
                                      associs  une frontire parallle */
  cs_int_t   *pos_per_fac_par;     /* Pointeurs sur blocs de cellules "fantmes"
                                      associs  une frontire parallle dans
                                      le cas d'une priodicit */
  cs_int_t   *num_cel_fac_par;     /* Numros locaux des cellules "fantmes" */
  cs_int_t   *num_dom_fac_par;     /* Numros de domaines sur frontire
                                      parallle */

  cs_param_perio_t *liste_param_per;   /* Paramtres de priodicit */

  /* Structures associes aux voisinages tendus */

  cs_int_t   *pos_dom_fac_par_voiset; /* Pointeurs sur blocs de cellules
                                         "fantmes" associs  une frontire
                                         parallle pour le voisinage
                                         tendu spar */
  cs_int_t   *pos_per_fac_par_voiset; /* Pointeurs sur blocs de cellules
                                         "fantmes" associs  une frontire
                                         parallle dans le cas d'une priodicit
                                         pour le voisinage tendu spar */
  cs_int_t   *num_cel_fac_par_voiset; /* Numros locaux des cellules "fantmes"
                                         pour le voisinage tendu spar  */
  cs_int_t   *num_dom_fac_par_voiset; /* Numros de domaines sur frontire
                                         parallle pour le voisinage
                                         tendu spar  */
  cs_real_t   *coord_cel_avec_voiset; /* Coordonnes du centre des cellules
                                         relles et "fantmes"
                                         pour le voisinage tendu spar  */

  cs_int_t  *pos_connect_voiset_cel_cel_dom;
                                      /* Position de la premire cellule dans la
                                         connectivit voisinage tendu : ipcvse */
  cs_int_t  *val_connect_voiset_cel_cel_dom;
                                      /* Numro local des cellules dans la
                                         connectivit voisinage tendu : ielvse */

  /* Historique des numrotations (pour changes et suites) */

  cs_int_t   *num_fac_ini;            /* Numrotation ini. des faces internes
                                         (si ces faces sont renumrotes) */
  cs_int_t   *num_fbr_ini;            /* Numrotation ini. des faces de bord
                                         (si ces faces sont renumrotes) */

  /* Rfrences des faces (groupes et familles) */

  cs_int_t    nbr_grp;                /* Nombre de groupes associs */
  cs_int_t   *pos_grp;                /* Position de dpart de chaque groupe
                                         dans le tableau des noms de groupe */
  char       *nom_grp;                /* Tableau des noms des groupes */

  cs_int_t    nbr_prop_fam_max;       /* Nombre de proprits maximal pour
                                         une famille */
  cs_int_t    nbr_fam;                /* Nombre de familles */

  cs_int_t   *prop_fam;               /* Proprits des familles */
  cs_int_t   *fam_cel;                /* Familles des cellules */
  cs_int_t   *fam_fbr;                /* Familles des faces de bord */

} cs_maillage_t ;


/* Structures de type "tampon" associes  un maillage */

#if defined(_CS_HAVE_MPI)

typedef struct {

  cs_int_t       nbr_dom_var_assembl;
                                   /* Nombre de domaines auxquels le processeur
                                      courant envoie de l'information        */
  cs_int_t       nbr_dom_var_assembl_voiset;
                                   /* Nombre de domaines auxquels le processeur
                                      courant envoie de l'information du
                                      voisinage tendu, s'il est trait
                                      sparment */

  MPI_Request   *mpi_request;      /* Tableau de requtes MPI */
  MPI_Status    *mpi_status;       /* Tableau de codes de retour MPI */

  cs_int_t      *num_cel_loc_par;  /* Numro local des cellules  envoyer */
  cs_int_t      *pos_dom_var_assembl;
                                   /* Reprage des cases destines aux
                                      diffrents voisins dans var_assembl */
  cs_int_t      *num_dom_var_assembl;
                                   /* Numro des domaines auxquels le processeur
                                      courant envoie de l'information  */

  cs_int_t      *num_cel_loc_par_voiset;
                                   /* Numro local des cellules  envoyer
                                      pour le voisinage tendu s'il est trait
                                      sparment */
  cs_int_t      *pos_dom_var_assembl_voiset;
                                   /* Reprage des cases destines aux
                                      diffrents voisins dans var_assembl
                                      pour le voisinage tendu s'il est trait
                                      sparment */
  cs_int_t      *num_dom_var_assembl_voiset;
                                   /* Numro des domaines auxquels le processeur
                                      courant envoie de l'information du
                                      voisinage tendu s'il est trait
                                      sparment */

  cs_real_t     *var_assembl;      /* Tableau d'assemblage temporaire */

  cs_real_t     *var_assembl_voiset;
                                   /* Tableau d'assemblage temporaire
                                      pour le voisinage tendu s'il est trait
                                      sparment */

  cs_real_t     *var_cel_avec_voiset;
                                   /* Tableau de rception pour
                                      le voisinage tendu s'il est trait
                                      sparment : longueur ncelet+voiset*/

} cs_maillage_tmp_t ;

#endif

/*=============================================================================
 * Variables globales_statiques
 *============================================================================*/

extern cs_maillage_t *cs_glob_maillage; /* Pointeur sur le maillage principal */

#if defined(_CS_HAVE_MPI)
extern cs_maillage_tmp_t *cs_glob_maillage_tmp ; /* Pointeur sur la structure
                                                    de tampons associe au
                                                    maillage principal */
#endif


/*============================================================================
 *  Fonctions publiques pour API Fortran
 *============================================================================*/

/*----------------------------------------------------------------------------
 * Recherche du numro de groupe correspondant  un nom donn. Si le groupe
 * existe, le numro renvoy correspond au rang du groupe (commenant  1)
 * dans la liste des groupes du maillage, et multipli par -1. Cette
 * numrotation est celle utilise dans le tableau IPRFML(NFML, NPRFML)
 * de description des familles.
 *
 * Si le groupe de nom indiqu n'existe pas, on renvoie 9999.
 *
 * Interface Fortran :
 *
 * FUNCTION NUMGRP (NOMGRP, LNGNOM)
 * ***************
 *
 * CHARACTER*       NOMGRP      : --> : Nom du groupe recherch
 * INTEGER          LNGNOM      : --> : Longueur du nom du groupe
 *----------------------------------------------------------------------------*/

cs_int_t CS_PROCF (numgrp, NUMGRP)
(
 const char       *const nomgrp,  /* --> Nom du groupe                        */
 const cs_int_t   *const lngnom   /* --> Longueur du nom                      */
 CS_ARGF_SUPP_CHAINE              /*     (arguments 'longueur' ventuels F77, */
                                  /*     inutiliss lors de l'appel mais      */
                                  /*     placs par de nombreux compilateurs) */
);


/*----------------------------------------------------------------------------
 * Sauvegarde des numrotations initiales des faces
 *
 * Remarque : lorsque des faces sont renumrotes, les connectivits et
 *            variables bases sur les faces sont toutes mises  jour, sauf
 *            les numros globaux associs  ces faces (en cas de paralllisme)
 *            et conservs dans la structure maillage ('num_fac'et num_fbr') ;
 *            en effet, cette numrotation est destine  se ramener  la
 *            numrotation initiale, notamment pour l'criture et la lecture de
 *            fichiers suite indpendants du nombre de domaines en paralllisme
 *            ou d'une renumrotation vectorielle, et il est plus ais de la
 *            manipuler si elle n'est pas modifie.
 *
 * Interface Fortran :
 *
 * SUBROUTINE SAVNUM (IVECTV, IVECTB, INUMFI, INUMFB)
 * *****************
 *
 * INTEGER          IVECTV      : --> : Indicateur renum. faces internes
 * INTEGER          IVECTB      : --> : Indicateur renum. faces de bord
 * INTEGER          INUMFI      : --> : Table de renum. des faces internes
 * INTEGER          INUMFB      : --> : Table de renum. des faces de bord
 *----------------------------------------------------------------------------*/

void CS_PROCF (savnum, SAVNUM)
(
 const cs_int_t   *const ivectv,  /* --> Indicateur renum. faces internes     */
 const cs_int_t   *const ivectb,  /* --> Indicateur renum. faces de bord      */
 const cs_int_t   *const inumfi,  /* --> Table de renum. des faces internes   */
 const cs_int_t   *const inumfb   /* --> Table de renum. des faces de bord    */
);


/*=============================================================================
 * Prototypes de fonctions
 *============================================================================*/

/*----------------------------------------------------------------------------
 * Cration d'une structure maillage
 *----------------------------------------------------------------------------*/

cs_maillage_t * cs_maillage_cree
(
  void
);


/*----------------------------------------------------------------------------
 * Destruction d'une structure maillage
 *----------------------------------------------------------------------------*/

cs_maillage_t * cs_maillage_detruit
(
  cs_maillage_t * maillage
);


/*----------------------------------------------------------------------------
 * Renumrotation des sommets de manire  assurer que :
 * si i < j, maillage->num_som[i] < maillage->num_som[j]
 * (ce qui n'est pas assur par la numrotation initialement
 * fournie par l'Enveloppe).
 *----------------------------------------------------------------------------*/

void cs_maillage_ordonne_sommets
(
 cs_maillage_t      *const maillage      /* <-> Structure maillage associe   */
);


/*----------------------------------------------------------------------------
 * Affichage d'informations relatives  un maillage
 *----------------------------------------------------------------------------*/

void cs_maillage_info
(
 const cs_maillage_t  *maillage
);


#if defined(_CS_HAVE_MPI)

/*----------------------------------------------------------------------------
 * Cration d'une structure de tampons associe  un maillage
 *----------------------------------------------------------------------------*/

cs_maillage_tmp_t * cs_maillage_tmp_cree
(
 cs_maillage_t  * maillage
);


/*----------------------------------------------------------------------------
 * Desctruction d'une structure de tampons associe  un maillage
 *----------------------------------------------------------------------------*/

cs_maillage_tmp_t * cs_maillage_tmp_detruit
(
 cs_maillage_tmp_t * maillage_tmp
);


/*----------------------------------------------------------------------------
 * Tableaux supplmentaires d'un maillage pour le paralllisme
 *----------------------------------------------------------------------------*/

void cs_maillage_init_parallel
(
 cs_maillage_t      *maillage,
 cs_maillage_tmp_t  *maillage_tmp
);


#endif /* _CS_HAVE_MPI */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __CS_MAILLAGE_H__ */
