/*============================================================================
*
*                    Code_Saturne version 1.3
*                    ------------------------
*
*
*     This file is part of the Code_Saturne Kernel, element of the
*     Code_Saturne CFD tool.
*
*     Copyright (C) 1998-2007 EDF S.A., France
*
*     contact: saturne-support@edf.fr
*
*     The Code_Saturne Kernel is free software; you can redistribute it
*     and/or modify it under the terms of the GNU General Public License
*     as published by the Free Software Foundation; either version 2 of
*     the License, or (at your option) any later version.
*
*     The Code_Saturne Kernel is distributed in the hope that it will be
*     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
*     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with the Code_Saturne Kernel; if not, write to the
*     Free Software Foundation, Inc.,
*     51 Franklin St, Fifth Floor,
*     Boston, MA  02110-1301  USA
*
*============================================================================*/

#ifndef __CS_MESSAGES_ECS_H__
#define __CS_MESSAGES_ECS_H__

/*============================================================================
 * Interfaces Fortran pour l'change de messages avec le module Enveloppe
 *============================================================================*/

/* includes systme */

#include <stdarg.h>


/* Includes librairie */

#include "cs_base.h"
#include "cs_maillage.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*============================================================================
 *  Fonctions publiques pour API Fortran
 *============================================================================*/

/*----------------------------------------------------------------------------
 * Dfinition pour la rception des messages 'dim' issus du Prprocesseur
 *
 * Interface Fortran :
 *
 * SUBROUTINE LEDEVI (NOMRUB, TYPENT, NBRENT, TABENT)
 * *****************
 *
 * INTEGER          NDIM        : <-- : Dimension de l'espace (3)
 * INTEGER          NCELET      : <-- : Nombre d'lments halo compris
 * INTEGER          NCEL        : <-- : Nombre d'lments actifs
 * INTEGER          NFAC        : <-- : Nombre de faces internes
 * INTEGER          NFABOR      : <-- : Nombre de faces de bord
 * INTEGER          NFML        : <-- : Nombre de familles des faces de bord
 * INTEGER          NPRFML      : <-- : Nombre de proprits max par famille
 * INTEGER          NSOM        : <-- : Nombre de sommets (optionnel)
 * INTEGER          LNDFAC      : <-- : Longueur de SOMFAC (optionnel)
 * INTEGER          LNDFBR      : <-- : Longueur de SOMFBR (optionnel)
 * INTEGER          IPERIO      : <-- : Indicateur de priodicit
 * INTEGER          IPEROT      : <-- : Nombre de priodicits de rotation
 *----------------------------------------------------------------------------*/

void CS_PROCF (ledevi, LEDEVI)
(
 cs_int_t   *const ndim,    /* <-- dimension de l'espace                      */
 cs_int_t   *const ncelet,  /* <-- nombre d'lments halo compris             */
 cs_int_t   *const ncel,    /* <-- nombre d'lments actifs                   */
 cs_int_t   *const nfac,    /* <-- nombre de faces internes                   */
 cs_int_t   *const nfabor,  /* <-- nombre de faces de bord                    */
 cs_int_t   *const nfml,    /* <-- nombre de familles des faces de bord       */
 cs_int_t   *const nprfml,  /* <-- nombre de proprits max par famille       */
 cs_int_t   *const nsom,    /* <-- nombre de sommets (optionnel)              */
 cs_int_t   *const lndfac,  /* <-- longueur de somfac (optionnel)             */
 cs_int_t   *const lndfbr,  /* <-- longueur de somfbr (optionnel)             */
 cs_int_t   *const iperio,  /* <-- indicateur de priodicit                  */
 cs_int_t   *const iperot   /* <-- nombre de priodicits de rotation         */
);


/*============================================================================
 * Prototypes de fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 * Rception des messages du bloc 'data' issus du module Enveloppe
 * puis fermeture de la communication.
 *----------------------------------------------------------------------------*/

void cs_messages_ecs_lit_data
(
 cs_maillage_t      *const maillage       /* <-> maillage associ             */
);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __CS_MESSAGES_ECS_H__ */

