/*============================================================================
*
*                    Code_Saturne version 1.3
*                    ------------------------
*
*
*     This file is part of the Code_Saturne Kernel, element of the
*     Code_Saturne CFD tool.
*
*     Copyright (C) 1998-2007 EDF S.A., France
*
*     contact: saturne-support@edf.fr
*
*     The Code_Saturne Kernel is free software; you can redistribute it
*     and/or modify it under the terms of the GNU General Public License
*     as published by the Free Software Foundation; either version 2 of
*     the License, or (at your option) any later version.
*
*     The Code_Saturne Kernel is distributed in the hope that it will be
*     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
*     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*     GNU General Public License for more details.
*
*     You should have received a copy of the GNU General Public License
*     along with the Code_Saturne Kernel; if not, write to the
*     Free Software Foundation, Inc.,
*     51 Franklin St, Fifth Floor,
*     Boston, MA  02110-1301  USA
*
*============================================================================*/

/*============================================================================
 * Interfaces Fortran pour la paralllisation du code
 *============================================================================*/

/* includes systme */


/* Includes BFT et FVM */

#include <fvm_interface.h>


/* Includes librairie */

#include "cs_base.h"


#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*============================================================================
 * Prototypes de fonctions publiques
 *============================================================================*/

/*----------------------------------------------------------------------------
 * Initialisations aprs lecture du maillage en cas de paralllisme
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARGEO
 * *****************
 *
 * INTEGER          NCELGB      : <-  : Nombre de cellules global
 * INTEGER          NFACGB      : <-  : Nombre de faces internes global
 * INTEGER          NFBRGB      : <-  : Nombre de faces internes global
 * INTEGER          NSOMGB      : <-  : Nombre de sommets global
 *----------------------------------------------------------------------------*/

void CS_PROCF (pargeo, PARGEO)
(
 cs_int_t  *ncelgb,             /* <-  Nombre de cellules global */
 cs_int_t  *nfacgb,             /* <-  Nombre de faces internes global */
 cs_int_t  *nfbrgb,             /* <-  Nombre de faces internes global */
 cs_int_t  *nsomgb              /* <-  Nombre de sommets global */
);


/*----------------------------------------------------------------------------
 * Mise  jour du voisinage tendu des coordonnes du centre des cellules
 *   dans le voisinage tendu s'il est trait sparment du reste du halo
 *
 * Le rle de cette fonction consiste  recopier les valeurs sur les
 * cellules principales en frontire parallle d'autres domaines
 * (indices entre 1 et ncel) vers les cellules halo du domaine en
 * cours et a les placer dans la structure maillage
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARGVE (NDIM, XYZCEN)
 * *****************
 *
 * INTEGER          NDIM             : --> : dimension de l'espace
 * DOUBLE PRECISION XYZCEN(3,NCELET) : --> : coordonnes du centre des cellules
 *----------------------------------------------------------------------------*/

void CS_PROCF (pargve, PARGVE)
(
 const cs_int_t  * const ndim      ,/* --> dimension de l'espace              */
 const cs_real_t * const xyzcen     /* --> coordonnes du centre des cellules */
);


/*----------------------------------------------------------------------------
 * Mise  jour d'un tableau sur les cellules en cas de paralllisme
 *
 * Le rle de cette fonction consiste  recopier les valeurs sur les
 * cellules principales en frontire parallle d'autres domaines
 * (indices entre 1 et ncel) vers les cellules halo du domaine en
 * cours (indices ncel+1  ncelet)
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARCOM (VAR)
 * *****************
 *
 * DOUBLE PRECISION VAR(NCELET) : <-> : variable sur les cellules, mise 
 *                                      jour de VAR(NCEL+1..NCELET) en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parcom, PARCOM)
(
 cs_real_t  var[]               /* <-> variable dfinie sur les cellules
                                 *     (mise  jour sur les cellules halo) */
);


/*----------------------------------------------------------------------------
 * Mise  jour du voisinage tendu d'une variable au centre des cellules
 *   dans le voisinage tendu s'il est trait sparment du reste du halo
 *
 * Fonction calque sur PARCOM
 *
 * Le rle de cette fonction consiste  recopier les valeurs sur les
 * cellules principales en frontire parallle d'autres domaines
 * (indices entre 1 et ncel) vers les cellules halo du domaine en
 * cours et a les placer dans la structure maillage
 *
 *----------------------------------------------------------------------------*/

void cs_parallel_parcve
(
 const cs_real_t * const var       ,/* --> variable au centre des cellules */
       cs_real_t *       var_voiset /* --> variable au centre des cellules
                                           dans le voisinage tendu        */
);


/*----------------------------------------------------------------------------
 * Interface FORTRAN pour les echanges sur le voisinage tendu lorsqu'il est
 *   trait sparment du halo classique. Le tableau trait est suppos
 *   correctement dimensionn (ncelet + nombre de cellules halo tendu).
 *
 * Appele en parallle uniquement.
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARCVE
 * *****************
 *
 *    & ( PVAR )
 *
 *----------------------------------------------------------------------------*/

void CS_PROCF (parcve, PARCVE)
(
       cs_real_t         pvar[]       /* <-> tab. echang                     */
 CS_ARGF_SUPP_CHAINE                  /*     (arguments 'longueur' ventuels,
                                             Fortran, inutiliss lors de
                                             l'appel mais placs par de
                                             nombreux compilateurs)           */
);


/*----------------------------------------------------------------------------
 * Maximum d'un compteur entier sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  mettre l'indicateur ind  la valeur
 * maximale obtenue sur les diffrents domaines de calcul
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARCMX (IND)
 * *****************
 *
 * INTEGER          IND         : <-> : indicateur local en entre,
 *                                      maximum en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parcmx, PARCMX)
(
 cs_int_t  *ind                 /* <-> indicateur */
);


/*----------------------------------------------------------------------------
 * Somme d'un compteur entier sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  ajouter au compteur cpt les
 * valeurs correspondantes sur les autre domaines de calcul
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARCPT (CPT)
 * *****************
 *
 * INTEGER          CPT         : <-> : compteur  sommer en entre,
 *                                      somme en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parcpt, PARCPT)
(
 cs_int_t  *cpt                 /* <-> compteur  sommer */
);


/*----------------------------------------------------------------------------
 * Somme d'un rel sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  ajouter au rel var les
 * valeurs correspondantes sur les autre domaines de calcul
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARSOM (VAR)
 * *****************
 *
 * DOUBLE PRECISION VAR         : <-> : valeur  sommer en entre,
 *                                      somme en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parsom, PARSOM)
(
 cs_real_t  *var                /* <-> variable  sommer */
);


/*----------------------------------------------------------------------------
 * Maximum d'un rel sur plusieurs domaines en cas de paralllisme
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARMAX (VAR)
 * *****************
 *
 * DOUBLE PRECISION VAR         : <-> : maximum local en entre,
 *                                      maximum global en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parmax, PARMAX)
(
 cs_real_t  *var
);


/*----------------------------------------------------------------------------
 * Minimum d'un rel sur plusieurs domaines en cas de paralllisme
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARMIN (VAR)
 * *****************
 *
 * DOUBLE PRECISION VAR         : <-> : minimum local en entre,
 *                                      minimum global en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parmin, PARMIN)
(
 cs_real_t  *var
);


/*----------------------------------------------------------------------------
 * Maximum d'un rel et valeurs associes sur plusieurs domaines
 * en cas de paralllisme
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARMXL (NBR, VAR, XYZVAR)
 * *****************
 *
 * INTEGER          NBR         :  -> : nombre de valeurs associees
 * DOUBLE PRECISION VAR         : <-> : maximum local en entre,
 *                                      maximum global en sortie
 * DOUBLE PRECISION XYZVAR(NBR) : <-> : val. associees au max loc.   en entre,
 *                                      val. associees au max. glob. en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parmxl, PARMXL)
(
 cs_int_t   *nbr,
 cs_real_t  *var,
 cs_real_t   xyzvar[]
);


/*----------------------------------------------------------------------------
 * Minimum d'un rel et valeurs associes sur plusieurs domaines
 * en cas de paralllisme
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARMNL (NBR, VAR, XYZVAR)
 * *****************
 *
 * INTEGER          NBR         :  -> : nombre de valeurs associees
 * DOUBLE PRECISION VAR         : <-> : minimum local en entre,
 *                                      minimum global en sortie
 * DOUBLE PRECISION XYZVAR(NBR) : <-> : valeurs associes min local en entre,
 *                                      valeurs associes min global en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parmnl, PARMNL)
(
 cs_int_t   *nbr,
 cs_real_t  *var,
 cs_real_t   xyzvar[]
);


/*----------------------------------------------------------------------------
 * Somme d'un tableau entier sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  mettre le tableau itab a la valeur
 * somme des valeurs correspondantes sur les autre domaines de calcul
 * (terme par terme)
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARISM (NBR,ITAB)
 * *****************
 *
 * INTEGER          NBR         :  -> : nombre de valeurs  envoyer/recevoir
 * INTEGER          ITAB(*)     : <-> : tableau local en entre,
 *                                      somme en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parism, PARISM)
(
 cs_int_t  *nbr,                 /* <-> dimension */
 cs_int_t   itab[]
);


/*----------------------------------------------------------------------------
 * Maximum d'un tableau entier sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  mettre le tableau ind  la valeur
 * maximale obtenue sur les diffrents domaines de calcul
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARIMX (NBR,ITAB)
 * *****************
 *
 * INTEGER          NBR         :  -> : nombre de valeurs  envoyer/recevoir
 * INTEGER          ITAB(*)     : <-> : tableau local en entre,
 *                                      maximum en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parimx, PARIMX)
(
 cs_int_t  *nbr,                 /* <-> dimension */
 cs_int_t   itab[]
);


/*----------------------------------------------------------------------------
 * Minimum d'un tableau entier sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  mettre le tableau itab  la valeur
 * minimale obtenue sur les diffrents domaines de calcul
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARIMN (NBR,ITAB)
 * *****************
 *
 * INTEGER          NBR         :  -> : nombre de valeurs  envoyer/recevoir
 * INTEGER          ITAB(*)     : <-> : tableau local en entre,
 *                                      minimum en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parimn, PARIMN)
(
 cs_int_t  *nbr,                 /* <-> dimension */
 cs_int_t   itab[]
);


  /*----------------------------------------------------------------------------
 * Somme d'un tableau reel sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  mettre le tableau rtab a la valeur
 * somme des valeurs correspondantes sur les autre domaines de calcul
 * (terme par terme)
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARRSM (NBR,RTAB)
 * *****************
 *
 * INTEGER          NBR         :  -> : nombre de valeurs  envoyer/recevoir
 * DOUBLE PRECISION RTAB(*)     : <-> : tableau local en entre,
 *                                      somme en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parrsm, PARRSM)
(
 cs_int_t  *nbr,                 /* <-> dimension */
 cs_real_t   rtab[]
);


/*----------------------------------------------------------------------------
 * Maximum d'un tableau reel   sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  mettre le tableau rtab  la valeur
 * maximale obtenue sur les diffrents domaines de calcul
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARRMX (NBR,RTAB)
 * *****************
 *
 * INTEGER          NBR         :  -> : nombre de valeurs  envoyer/recevoir
 * DOUBLE PRECISION RTAB(*)     : <-> : tableau local en entre,
 *                                      maximum en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parrmx, PARRMX)
(
 cs_int_t  *nbr,                 /* <-> dimension */
 cs_real_t   rtab[]
);


/*----------------------------------------------------------------------------
 * Minimum d'un tableau reel   sur plusieurs domaines en cas de paralllisme
 *
 * Le rle de cette fonction consiste  mettre le tableau rtab  la valeur
 * minimale obtenue sur les diffrents domaines de calcul
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARRMN (NBR,RTAB)
 * *****************
 *
 * INTEGER          NBR         :  -> : nombre de valeurs  envoyer/recevoir
 * INTEGER          RTAB(*)     : <-> : tableau local en entre,
 *                                      minimum en sortie
 *----------------------------------------------------------------------------*/

void CS_PROCF (parrmn, PARRMN)
(
 cs_int_t  *nbr,                 /* <-> dimension */
 cs_real_t   rtab[]
);


/*----------------------------------------------------------------------------
 * Envoi de valeurs entires d'un processus  ses voisins
 * (encapsulation de MPI_Bcast())
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARBCI (IRANGV, NBR, ITAB)
 * *****************
 *
 * INTEGER          IRANGV      :  -> : rang du processus associ aux valeurs
 *                                       envoyer
 * INTEGER          NBR         :  -> : nombre de valeurs  envoyer/recevoir
 * DOUBLE PRECISION ITAB(*)     : <-> : valeurs entires
 *----------------------------------------------------------------------------*/

void CS_PROCF (parbci, PARBCI)
(
 cs_int_t   *irangv,
 cs_int_t   *nbr,
 cs_int_t    itab[]
);


/*----------------------------------------------------------------------------
 * Envoi de valeurs relles d'un processus  ses voisins
 * (encapsulation de MPI_Bcast())
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARBCR (IRANGV, NBR, RTAB)
 * *****************
 *
 * INTEGER          IRANGV      :  -> : rang du processus associ aux valeurs
 *                                       envoyer
 * INTEGER          NBR         :  -> : nombre de valeurs  envoyer/recevoir
 * DOUBLE PRECISION RTAB(*)     : <-> : valeurs relles
 *----------------------------------------------------------------------------*/

void CS_PROCF (parbcr, PARBCR)
(
 cs_int_t   *irangv,
 cs_int_t   *nbr,
 cs_real_t   rtab[]
);

/*----------------------------------------------------------------------------
 * Mthode "AllGather" pour rcuprer les donnes sur tous les processeurs
 * dans le cas o le nombre de variables locales est diffrent suivant
 * les domaines
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARAGV (NVAR, NVARGB, VAR, VARGB)
 * *****************
 *
 * INTEGER          NVAR        :  -> : nombre de variables locales
 * INTEGER          NVARGB      :  -> : nombre de variables globales
 * DOUBLE PRECISION VAR(*)      :  -> : variables locales
 * DOUBLE PRECISION VARGB(*)    : <-  : variables globales
 *----------------------------------------------------------------------------*/

void CS_PROCF (paragv, PARAGV)
(
 cs_int_t   *nvar,
 cs_int_t   *nvargb,
 cs_real_t   var[],
 cs_real_t  *vargb
);


/*----------------------------------------------------------------------------
 * Recherche du point minimisant une distance donne et du rang associ au
 * processus contenant ce point (pour localiser un point parmi plusieurs
 * domaines)
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARFPT (NODE, NDRANG, DIS2MN, XYZSND)
 * *****************
 *
 * INTEGER          NODE        : <-> : numro local du point le plus proche
 *                                      du point recherch
 * INTEGER          NDRANG      : <-  : rang du processus pour lequel la
 *                                      distance est la plus faible
 * DOUBLE PRECISION DIS2MN      :  -> : distance au carr du point en entre
 *                                      au point recherch
 *----------------------------------------------------------------------------*/

void CS_PROCF (parfpt, PARFPT)
(
 cs_int_t   *node,
 cs_int_t   *ndrang,
 cs_real_t  *dis2mn
);


/*----------------------------------------------------------------------------
 * Renvoi d'une valeur associe  une sonde
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARHIS (NODE, NDRANG, VAR, VARCAP)
 * *****************
 *
 * INTEGER          NODE        :  -> : numro local de l'lment associ
 *                                       la sonde
 * INTEGER          NDRANG      :  -> : rang du processus associ  la sonde
 *                                      distance est la plus faible
 * DOUBLE PRECISION VAR(*)      :  -> : variable discrtise
 * DOUBLE PRECISION VARCAP      : <-  : valeur de la variable au point associ
 *                                       la sonde
 *----------------------------------------------------------------------------*/

void CS_PROCF (parhis, PARHIS)
(
 cs_int_t   *node,
 cs_int_t   *ndrang,
 cs_real_t   var[],
 cs_real_t  *varcap
);


/*----------------------------------------------------------------------------
 * Numro de cellule global associ  un numro de cellule local
 *   (envoi a tous les processeurs)
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARCEL (NUMLOC,INDDOM,NUMGLO)
 * *****************
 *
 * INTEGER          NUMLOC      :  -> : numro de cellule local
 * INTEGER          INDDOM      :  -> : rang du domaine associ (0  N-1)
 * INTEGER          NUMGLO      :  -> : numro de cellule global
 *----------------------------------------------------------------------------*/

void CS_PROCF (parcel, PARCEL)
(
 cs_int_t   *numloc,
 cs_int_t   *inddom,
 cs_int_t   *numglo
);


/*----------------------------------------------------------------------------
 * Numro de cellule global associ  un numro de cellule local
 *   du processeur courant, sans communication avec les autres
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARCLG (NUMLOC,INDDOM,NUMGLO)
 * *****************
 *
 * INTEGER          NUMLOC      :  -> : numro de cellule local
 * INTEGER          INDDOM      :  -> : rang du domaine associ (0  N-1)
 * INTEGER          NUMGLO      :  -> : numro de cellule global
 *----------------------------------------------------------------------------*/

void CS_PROCF (parclg, PARCLG)
(
 cs_int_t   *numloc,
 cs_int_t   *inddom,
 cs_int_t   *numglo
);


/*----------------------------------------------------------------------------
 * Numro de face interne global associ  un numro de face interne local
 *   du processeur courant, sans communication avec les autres
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARFIG (NUMLOC,INDDOM,NUMGLO)
 * *****************
 *
 * INTEGER          NUMLOC      :  -> : numro de face interne local
 * INTEGER          INDDOM      :  -> : rang du domaine associ (0  N-1)
 * INTEGER          NUMGLO      :  -> : numro de face interne global
 *----------------------------------------------------------------------------*/

void CS_PROCF (parfig, PARFIG)
(
 cs_int_t   *numloc,
 cs_int_t   *inddom,
 cs_int_t   *numglo
);


/*----------------------------------------------------------------------------
 * Numro de face de bord global associ  un numro de face de bord local
 *   du processeur courant, sans communication avec les autres
 *
 * Interface Fortran :
 *
 * SUBROUTINE PARFBG (NUMLOC,INDDOM,NUMGLO)
 * *****************
 *
 * INTEGER          NUMLOC      :  -> : numro de face de bord local
 * INTEGER          INDDOM      :  -> : rang du domaine associ (0  N-1)
 * INTEGER          NUMGLO      :  -> : numro de face de bord global
 *----------------------------------------------------------------------------*/

void CS_PROCF (parfbg, PARFBG)
(
 cs_int_t   *numloc,
 cs_int_t   *inddom,
 cs_int_t   *numglo
);


/*----------------------------------------------------------------------------
 * Somme aux entits correspondant  une interface interdomaines
 * des valeurs d'une variable relle.
 *
 * Les valeurs aux entits ne correspondant pas  l'interface sont inchanges,
 * celles aux entits correspondant  l'interface sont sommes avec les
 * valeurs distantes associes.
 *----------------------------------------------------------------------------*/

void cs_parallel_interface_sr
(
 fvm_interface_set_t  *interfaces,     /* --> jeu d'interfaces interdomaines  */
 cs_int_t              nbr_ent,        /* --> nombre d'entits associes      */
 cs_int_t              pas,            /* --> nombre de valeurs (non
                                        *     entrelaces)  par entit        */
 cs_real_t            *var             /* <-> variable                        */
);


/*----------------------------------------------------------------------------*/
#ifdef __cplusplus
}
#endif /* __cplusplus */

